@@ -26,6 +26,7 @@ if sys.platform == "win32":
 import localization as lang
 from log import safe_print as log_safe_print
 from meta import name as appname, version
+from network import get_network_addr, get_valid_host
 from ordereddict import OrderedDict
 from util_str import safe_str, safe_unicode
 
@@ -35,7 +36,7 @@ CALLBACK = ctypes.CFUNCTYPE(None, ctypes.c_void_p, ctypes.POINTER(None),
 							ctypes.c_char_p, ctypes.c_ulonglong, ctypes.c_bool)
 
 
-min_version = (0, 87, 14, 0)
+min_version = (0, 88, 14, 0)
 
 
 # Search for madTPG on the local PC, connect to the first found instance
@@ -52,12 +53,12 @@ CM_ShowIpAddrDialog = 4
 CM_Fail = 5
 
 
-_methodnames = ("ConnectEx", "Disable3dlut", "Enable3dlut",
+_methodnames = ("ConnectEx", "Disable3dlut", "Enable3dlut", "EnterFullscreen",
 				"GetBlackAndWhiteLevel", "GetDeviceGammaRamp",
 				"GetSelected3dlut", "GetVersion",
-				"IsDisableOsdButtonPressed",
+				"IsDisableOsdButtonPressed", "IsFullscreen",
 				"IsStayOnTopButtonPressed",
-				"IsUseFullscreenButtonPressed",
+				"IsUseFullscreenButtonPressed", "LeaveFullscreen",
 				"SetDisableOsdButton",
 				"SetDeviceGammaRamp", "SetOsdText",
 				"GetPatternConfig", "SetPatternConfig",
@@ -110,6 +111,7 @@ class H3DLUT(object):
 	# https://sourceforge.net/projects/thr3dlut
 
 	def __init__(self, filename):
+		self.fileName = filename
 		with open(filename, "rb") as lut:
 			data = lut.read()
 		self.signature = data[:4]
@@ -121,7 +123,7 @@ class H3DLUT(object):
 		self.outputBitDepth = struct.unpack("<l", data[64:68])[0]
 		self.outputColorEncoding = struct.unpack("<l", data[68:72])[0]
 		self.parametersFileOffset = struct.unpack("<l", data[72:76])[0]
-		self.parametersSize = struct.unpack("<l", data[76:80])[0]
+		parametersSize = struct.unpack("<l", data[76:80])[0]
 		self.lutFileOffset = struct.unpack("<l", data[80:84])[0]
 		self.lutCompressionMethod = struct.unpack("<l", data[84:88])[0]
 		if self.lutCompressionMethod != 0:
@@ -131,7 +133,7 @@ class H3DLUT(object):
 		self.lutUncompressedSize = struct.unpack("<l", data[92:96])[0]
 		self.parametersData = data[self.parametersFileOffset:
 								   self.parametersFileOffset +
-								   self.parametersSize]
+								   parametersSize]
 		self.LUTDATA = data[self.lutFileOffset:
 							self.lutFileOffset + self.lutCompressedSize]
 		if len(self.LUTDATA) != self.lutCompressedSize:
@@ -142,15 +144,75 @@ class H3DLUT(object):
 			self.LUTDATA += data[self.lutFileOffset + self.lutCompressedSize:
 								 self.lutFileOffset + self.lutCompressedSize +
 								 1552]
-			self.lutCompressedSize += 1552
-			self.lutUncompressedSize += 1552
+
+	@property
+	def data(self):
+		return "".join((self.signature,
+						struct.pack("<l", self.fileVersion),
+						self.programName.ljust(32, "\0"),
+						struct.pack("<q", self.programVersion),
+						struct.pack(*("<3l",) + self.inputBitDepth),
+						struct.pack("<l", self.inputColorEncoding),
+						struct.pack("<l", self.outputBitDepth),
+						struct.pack("<l", self.outputColorEncoding),
+						struct.pack("<l", self.parametersFileOffset),
+						struct.pack("<l", self.parametersSize),
+						struct.pack("<l", self.lutFileOffset),
+						struct.pack("<l", self.lutCompressionMethod),
+						struct.pack("<l", self.lutCompressedSize),
+						struct.pack("<l", self.lutUncompressedSize),
+						"\0" * (self.parametersFileOffset - 96),
+						self.parametersData,
+						"\0" * (self.lutFileOffset - self.parametersFileOffset - self.parametersSize),
+						self.LUTDATA))
+
+	@property
+	def parametersSize(self):
+		return len(self.parametersData)
+
+	def write(self, stream_or_filename=None):
+		"""
+		Write 3D LUT to stream or filename.
+		
+		"""
+		if not stream_or_filename:
+			stream_or_filename = self.fileName
+		if isinstance(stream_or_filename, basestring):
+			stream = open(stream_or_filename, "wb")
+			if not self.fileName:
+				self.fileName = stream_or_filename
+		else:
+			stream = stream_or_filename
+		stream.write(self.data)
+		if isinstance(stream_or_filename, basestring):
+			stream.close()
 
 
-class MadTPG(object):
+class MadTPGBase(object):
+
+	""" Generic pattern generator compatibility layer """
+
+	def wait(self):
+		self.connect(method2=CM_StartLocalInstance)
+
+	def disconnect_client(self):
+		self.disconnect()
+
+	def send(self, rgb=(0, 0, 0), bgrgb=(0, 0, 0), bits=None,
+			 use_video_levels=None, x=0, y=0, w=1, h=1):
+		cfg = self.get_pattern_config()
+		self.set_pattern_config(int(round((w + h) / 2.0 * 100)),
+								int(round(sum(bgrgb) / 3.0 * 100)), cfg[2],
+								cfg[3])
+		self.show_rgb(*rgb + bgrgb)
+
+
+class MadTPG(MadTPGBase):
 
 	""" Minimal madTPG controller class """
 
 	def __init__(self):
+		MadTPGBase.__init__(self)
 		self._connection_callbacks = []
 
 		# We only expose stuff we might actually use.
@@ -289,7 +351,7 @@ class MadTPG(object):
 		return self.dllpath
 
 
-class MadTPG_Net(object):
+class MadTPG_Net(MadTPGBase):
 
 	""" Implementation of madVR network protocol in pure python """
 
@@ -297,13 +359,17 @@ class MadTPG_Net(object):
 	# (tcp.dstport != 1900 and tcp.dstport != 443) or (udp.dstport != 1900 and udp.dstport != 137 and udp.dstport != 138 and udp.dstport != 5355 and udp.dstport != 547 and udp.dstport != 10111)
 
 	def __init__(self):
+		MadTPGBase.__init__(self)
 		self._cast_sockets = {}
 		self._casts = []
 		self._client_sockets = OrderedDict()
 		self._commandno = 0
 		self._commands = {}
-		hostname = socket.gethostname()
-		self._host = socket.gethostbyname(hostname)
+		self._host = get_network_addr()
+		try:
+			hostname = get_valid_host()[0]
+		except socket.error:
+			hostname = self._host
 		self._incoming = {}
 		self._ips = [i[4][0] for i in socket.getaddrinfo(hostname, None)]
 		self._pid = 0
